using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using TimeTag;


namespace TimetagDemo
{
    public partial class FormTimetagDemo : Form
    {
        public FormTimetagDemo()
        {
            InitializeComponent();
        }
        TTInterface ttInterface;

//This code is executed when the Open Button is pressed
private void ButtonOpen_Click(object sender, EventArgs e)
{
    try
    {
        ttInterface = new TTInterface();  //Create object
        ttInterface.Open(1);              //Open it. Always 1 when only one device connected
        EnableGui(true);
        ShowStatus();
    }
    catch (Exception ex )
    {
        MessageBox.Show("Can't open device: "+ex.Message);
        return;
    }
}
private void ShowStatus()
{
    try
    {
        richTextBox1.AppendText("FPGA Version:\t" + ttInterface.GetFpgaVersion()/100.0 + "\n");
        richTextBox1.AppendText("Resolution/ps:\t" + ttInterface.GetResolution() * 1e12 + "\n");
        richTextBox1.AppendText("Errors:\t\t" + ttInterface.GetErrorText(ttInterface.ReadErrorFlags()) + "\n\n");
    }
    catch (Exception ex)
    {
        richTextBox1.AppendText("Error: " + ex.Message);
    }
}


        private void EnableGui(bool ena)
        {

            foreach (Control c in Controls)
            {
                if (c == ButtonOpen)
                    c.Enabled = !ena;
                else
                    c.Enabled = ena;
            
            }
            
        }
        
//This code is executed regularly
private void timer1_Tick(object sender, EventArgs e)
{
    try
    {
        if (ttInterface != null && ttInterface.IsOpen())
        {
            richTextBox1.Clear();
            ShowStatus();

            if (checkBoxReadTags.Checked)
            {
                DisplayTags();
            }
        }
    }
    catch (Exception ex)
    {
        richTextBox1.AppendText(ex.Message + "\n");
    }

}
   
      
        
        private void DisplayTags()
        {
            byte[] channels;
            long[] times;

            int count = ttInterface.ReadTags(out channels, out times);

            if (count == 0)
                richTextBox1.AppendText("No Tags\n");  
            else
            {
                int displayCount = Math.Min(count, 100); //Not all data is displayd for speed reasons
                long OldTime = 0;

                for (int i = 0; i < displayCount; i++)
                {
                    long time = times[i];
                    long timeDiff = time - OldTime;
                    OldTime = time;
                    double ns = timeDiff * ttInterface.GetResolution() * 1e9;
                    if (i > 1)
                        richTextBox1.AppendText("Channel: " + channels[i] + "  TimeDiff [ns]: " + ns + "\n");
                }
            }
        }

   

        private void FormTimetagDemo_Load(object sender, EventArgs e)
        {
            EnableGui(false);
        }
        
        //This code starts / stops transmission
        private void checkBoxReadTags_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBoxReadTags.Checked)
                ttInterface.StartTimetags();
            else
                ttInterface.StopTimetags();
        }
      
        


        private void buttonCalibrate_Click(object sender, EventArgs e)
        {
            checkBoxReadTags.Checked = false;
            
            richTextBox1.Clear();
            richTextBox1.AppendText("Calibration running\n");
            Application.DoEvents();
            ttInterface.Calibrate();
            richTextBox1.AppendText("Calibration completed");
            
            
        }


        private void FormTimetagDemo_FormClosed(object sender, FormClosedEventArgs e)

        {
            try
            {
                if (ttInterface != null)
                    ttInterface.Close();

            }
            catch (Exception)
            { 
            }
        }
       

        private void checkBoxUse10MHz_CheckedChanged(object sender, EventArgs e)
        {
            ttInterface.Use10MHz((sender as CheckBox).Checked);
        }
    }
}